import { defineComponent, shallowRef, computed, onUnmounted, watch, createVNode, onMounted, nextTick, ref } from 'vue';
import useMonaco from '../../hooks/useMonaco.js';
import { editorProps } from './types.js';
import { getOrCreateModel, isUndefined } from '../../utils/index.js';
import MonacoContainer from '../monacoContainer/MonacoContainer.js';

var Editor = defineComponent({
  props: editorProps,

  setup(props, _ref) {
    let {
      slots
    } = _ref;
    const viewStates = new Map();
    const containerRef = shallowRef(null);

    const setContainerRef = el => containerRef.value = el;

    const {
      monacoRef,
      unload
    } = useMonaco();
    const {
      editorRef
    } = useEditor(props, monacoRef, containerRef);
    const {
      disposeValidator
    } = useValidator(props, monacoRef, editorRef);
    const isEditorReady = computed(() => !!monacoRef.value && !!editorRef.value);
    onUnmounted(() => {
      var _disposeValidator$val, _editorRef$value, _editorRef$value$getM, _editorRef$value2;

      (_disposeValidator$val = disposeValidator.value) === null || _disposeValidator$val === void 0 ? void 0 : _disposeValidator$val.call(disposeValidator);
      (_editorRef$value = editorRef.value) === null || _editorRef$value === void 0 ? void 0 : (_editorRef$value$getM = _editorRef$value.getModel()) === null || _editorRef$value$getM === void 0 ? void 0 : _editorRef$value$getM.dispose();
      ((_editorRef$value2 = editorRef.value) === null || _editorRef$value2 === void 0 ? void 0 : _editorRef$value2.dispose()) ?? unload();
    }); // value

    watch(() => props.value, newValue => {
      if (editorRef.value && editorRef.value.getValue() !== newValue) {
        editorRef.value.setValue(newValue);
      }
    }); // path

    watch(() => props.path, (newPath, oldPath) => {
      const model = getOrCreateModel(monacoRef.value, props.value || props.defaultValue, props.language || props.defaultLanguage, newPath);

      if (model !== editorRef.value.getModel()) {
        props.saveViewState && viewStates.set(oldPath, editorRef.value.saveViewState());
        editorRef.value.setModel(model);
        props.saveViewState && editorRef.value.restoreViewState(viewStates.get(newPath));
      }
    }); // options

    watch(() => props.options, options => editorRef.value && editorRef.value.updateOptions(options), {
      deep: true
    }); // theme

    watch(() => props.theme, theme => monacoRef.value && monacoRef.value.editor.setTheme(theme)); // language

    watch(() => props.language, language => isEditorReady.value && monacoRef.value.editor.setModelLanguage(editorRef.value.getModel(), language)); // line

    watch(() => props.line, line => {
      // reason for undefined check: https://github.com/suren-atoyan/monaco-react/pull/188
      if (editorRef.value && !isUndefined(line)) {
        editorRef.value.revealLine(line);
      }
    });
    return () => {
      var _slots$default;

      return createVNode(MonacoContainer, {
        "setContainerRef": setContainerRef,
        "width": props.width,
        "height": props.height,
        "isEditorReady": isEditorReady.value
      }, {
        default: () => [((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)) ?? 'loading...']
      });
    };
  }

});

function useEditor(props, monacoRef, containerRef) {
  const editorRef = shallowRef(null);
  onMounted(() => {
    const stop = watch(monacoRef, () => {
      if (containerRef.value && monacoRef.value) {
        nextTick(() => stop());
        createEditor();
      }
    }, {
      immediate: true
    });
  });

  function createEditor() {
    var _props$onBeforeMount, _editorRef$value3, _props$onMount;

    if (!containerRef.value || !monacoRef.value || editorRef.value) {
      return;
    } // editor before mount


    (_props$onBeforeMount = props.onBeforeMount) === null || _props$onBeforeMount === void 0 ? void 0 : _props$onBeforeMount.call(props, monacoRef.value);
    const autoCreatedModelPath = props.path || props.defaultPath;
    const defaultModel = getOrCreateModel(monacoRef.value, props.value || props.defaultValue, props.language || props.defaultLanguage, autoCreatedModelPath);
    editorRef.value = monacoRef.value.editor.create(containerRef.value, {
      model: defaultModel,
      theme: props.theme,
      automaticLayout: true,
      autoIndent: 'brackets',
      formatOnPaste: true,
      formatOnType: true,
      ...props.options
    }, props.overrideServices);
    (_editorRef$value3 = editorRef.value) === null || _editorRef$value3 === void 0 ? void 0 : _editorRef$value3.onDidChangeModelContent(event => {
      const value = editorRef.value.getValue();

      if (value !== props.value) {
        var _props$onUpdateValue, _props$onChange;

        (_props$onUpdateValue = props['onUpdate:value']) === null || _props$onUpdateValue === void 0 ? void 0 : _props$onUpdateValue.call(props, value);
        (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, value, event);
      }
    }); // editor mount

    (_props$onMount = props.onMount) === null || _props$onMount === void 0 ? void 0 : _props$onMount.call(props, editorRef.value, monacoRef.value);
  }

  return {
    editorRef
  };
}

function useValidator(props, monacoRef, editorRef) {
  const disposeValidator = ref(null);
  const stop = watch([monacoRef, editorRef], () => {
    if (monacoRef.value && editorRef.value) {
      nextTick(() => stop());
      const changeMarkersListener = monacoRef.value.editor.onDidChangeMarkers(uris => {
        var _editorRef$value4, _editorRef$value4$get;

        const editorUri = (_editorRef$value4 = editorRef.value) === null || _editorRef$value4 === void 0 ? void 0 : (_editorRef$value4$get = _editorRef$value4.getModel()) === null || _editorRef$value4$get === void 0 ? void 0 : _editorRef$value4$get.uri;

        if (editorUri) {
          const currentEditorHasMarkerChanges = uris.find(uri => uri.path === editorUri.path);

          if (currentEditorHasMarkerChanges) {
            var _props$onValidate;

            const markers = monacoRef.value.editor.getModelMarkers({
              resource: editorUri
            });
            (_props$onValidate = props.onValidate) === null || _props$onValidate === void 0 ? void 0 : _props$onValidate.call(props, markers);
          }
        }
      });

      disposeValidator.value = () => changeMarkersListener === null || changeMarkersListener === void 0 ? void 0 : changeMarkersListener.dispose();
    }
  });
  return {
    disposeValidator
  };
}

export { Editor as default };
